package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.community.commom.hk.HKOpenApi;
import com.community.commom.mode.ResponseBean;
import com.community.commom.uuid.IdGen;
import com.community.huiju.dao.*;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.*;
import com.community.huiju.service.IHKService;
import com.community.huiju.service.ISenCartId;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.bouncycastle.cms.PasswordRecipientId;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * weiximei on 2019-06-21
 */
@Service
@Slf4j
public class SenCartIdImpl implements ISenCartId {

    @Autowired
    private IHKService ihkService;

    @Autowired
    private TaUserMapper taUserMapper;

    @Autowired
    private TpUnitHkSettingMapper tpUnitHkSettingMapper;


    @Autowired
    private TpEquipmentTreeMapper tpEquipmentTreeMapper;

    private IdGen idGen = IdGen.get();

    @Autowired
    private TaUserHkMapper taUserHkMapper;

    @Autowired
    private TpEquipmentMapper tpEquipmentMapper;

    @Autowired
    private TpConfigurationMapper tpConfigurationMapper;

    @Override
    public ResponseBean addUserAndAddCartId(Integer communityId) {
        ResponseBean responseBean = new ResponseBean();

        // 查询审核已通过的，但是没有海康id的
        List<TaUser> taUserList = taUserMapper.selectNotHKuserId(communityId);

        for (TaUser user : taUserList) {

            // 海康权限信息
            TpUnitHkSetting unitHkSetting = getUnitHkSetting(communityId, user.getUnitId());

            TpEquipmentTree tpEquipmentTree = tpEquipmentTreeMapper.selectByCommunityId(communityId);

            TaUserHk taUserHk = new TaUserHk();
            // 设置 海康id
            taUserHk.setHkPersonNo(idGen.nextId());
            /**
             * 1.获取部门(比如 住户)
             * 2.根据部门编号推送 海康
             */
            taUserMapper.updateByPrimaryKeySelective(user);
            addUserAndOpenCard(responseBean, user, tpEquipmentTree, taUserHk, unitHkSetting);


            // 卡片操作

            // 添加卡片
            responseBean = addCards(taUserHk, tpEquipmentTree);

            // 开卡
            responseBean = openCard(taUserHk.getHkUserId(), Long.valueOf(taUserHk.getHkCardNo()), null,tpEquipmentTree);

        }

        responseBean.addSuccess("操作成功！");
        return responseBean;
    }


    /**
     * 海康权限组
     * @return
     */
    private TpUnitHkSetting getUnitHkSetting(Integer communityId, Integer unitId) {
        TpUnitHkSetting tpUnitHkSetting = tpUnitHkSettingMapper.selectByCommunityIdAndUnitId(communityId, unitId);
        if (null == tpUnitHkSetting) {
            throw new WisdomException("权限组未配置");
        }
        return tpUnitHkSetting;
    }

    /**
     * 公共方法
     * 添加人员
     *
     * @param response
     * @param user
     */
    private void addUserAndOpenCard(ResponseBean response, TaUser user, TpEquipmentTree tpEquipmentTree, TaUserHk taUserHk, TpUnitHkSetting unitHkSetting) {
//        Map<String, Object> parDept = Maps.newHashMap();
//        parDept.put("pageNo", 1);
//        parDept.put("pageSize", 100);
//        parDept.put("deptName", Constant.DEPT_RESIDENTS);
//        parDept.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
//        parDept.put("appkey", tpEquipmentTree.getAppkey());
//        parDept.put("secret", tpEquipmentTree.getSecret());
//        parDept.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
        // 部门UUID
        //String deptUuid = getDeptUUID(parDept);

        // 添加人员
        Map<String, Object> parUser = Maps.newHashMap();
        parUser.put("personNo", taUserHk.getHkPersonNo());
        parUser.put("personName", user.getUserName());
        // parUser.put("phoneNo", user.getLoginName());
        parUser.put("remark", user.getLoginName());
        parUser.put("deptUuid", unitHkSetting.getDepartmentId());
        parUser.put("secret", tpEquipmentTree.getSecret());
        parUser.put("appkey", tpEquipmentTree.getAppkey());
        parUser.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
        parUser.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
        Map<String, Object> resultMap = JSONObject.parseObject(HKOpenApi.addUser(parUser), HashMap.class);
        Integer errorCode = (Integer) resultMap.get("errorCode");
        if (errorCode == 0) {
            Map<String, Object> resultDataMap = (Map<String, Object>) resultMap.get("data");

            // 海康人员ID
            Integer personId = (Integer) resultDataMap.get("personId");

            taUserHk.setHkUserId(personId);

            int row = 0;
            if (null == taUserHk.getId()) {
                taUserHk.setTaUserId(user.getId());
                taUserHk.setCommunityId(tpEquipmentTree.getCommunityId());
                // 存储海康人员ID
                row = taUserHkMapper.insertSelective(taUserHk);
            } else {
                row = taUserHkMapper.updateByPrimaryKeySelective(taUserHk);
            }

            if (row > 0) {
                response.addSuccess("操作成功!");
            } else {
                response.addError("操作失败");
                throw new RuntimeException("数据库添加 家属/租客 人员失败!");
            }

        } else {
            String errorMessage = String.valueOf(resultMap.get("errorMessage"));
            response.addError(errorMessage);
            log.error("海康添加人员失败! {}", errorMessage);
            throw new RuntimeException(errorMessage);
        }
    }

    /**
     * 添加卡片
     * @return
     */
    public ResponseBean addCards(TaUserHk taUserHk, TpEquipmentTree tpEquipmentTree) {

        ResponseBean response = new ResponseBean();

        // 开卡 卡号
        long cardNo = System.currentTimeMillis();

        // 添加卡片
        Map<String,Object> addCardsMap = new HashMap<>();
        addCardsMap.put("secret", tpEquipmentTree.getSecret());
        addCardsMap.put("appkey", tpEquipmentTree.getAppkey());
        addCardsMap.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
        addCardsMap.put("startCardNo", cardNo);
        addCardsMap.put("endCardNo", cardNo);
        addCardsMap.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
        String addCardsJSON = HKOpenApi.addCards(addCardsMap);
        JSONObject addCardsJsonObject = JSONObject.parseObject(addCardsJSON);
        Integer addCardsResultCode = addCardsJsonObject.getInteger("errorCode");
        if (0 != addCardsResultCode) {
            response.addError("操作失败");
            throw new RuntimeException(addCardsJsonObject.getString("errorMessage"));
        }

        // 卡片入库
        taUserHk.setHkCardNo(cardNo + "");
        taUserHkMapper.updateByPrimaryKeySelective(taUserHk);
        return response;
    }

    /**
     * 卡片开卡
     *
     * @param personId 人员海康id
     * @param cardNo 卡号
     * @param expirationTime    有效期为多少? 单位为 年
     * @return
     */
    private ResponseBean openCard(Integer personId, Long cardNo, Integer expirationTime, TpEquipmentTree tpEquipmentTree) {
        ResponseBean response = new ResponseBean();

        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        LocalDate today = LocalDate.now();
        int year = today.getYear();
        // 加上20, 表示加上20年,默认
        if (null == expirationTime || expirationTime <= 0) {
            year += 20;
        } else {
            year += expirationTime;
        }
        int month = today.getMonthValue();
        int day = today.getDayOfMonth();

        Long time = null;
        try {
            time = formatter.parse(year + "-" + month + "-" + day).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        // 开卡
        Map<String, Object> parOpenCard = Maps.newHashMap();
        parOpenCard.put("personId", personId);
        parOpenCard.put("cardNo", cardNo);
        parOpenCard.put("startTime", System.currentTimeMillis());
        parOpenCard.put("endTime", time);
        parOpenCard.put("secret", tpEquipmentTree.getSecret());
        parOpenCard.put("appkey", tpEquipmentTree.getAppkey());
        parOpenCard.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
        parOpenCard.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
        JSONObject openCardJSON = JSONObject.parseObject(HKOpenApi.openCard(parOpenCard));
        int code = (int) openCardJSON.get("errorCode");
        if (code == 0) {
            response.addSuccess("操作成功!");
        } else {
            String errorMessage = String.valueOf(openCardJSON.get("errorMessage"));
            response.addError(errorMessage);
            log.error("开卡失败! {}", errorMessage);
            throw new RuntimeException(errorMessage);
        }

        return response;
    }

    @Override
    public ResponseBean addUserOne(Integer communityId, MultipartFile uploadFile) {
        ResponseBean responseBean = new ResponseBean();


        TaUser user = new TaUser();
        user.setLoginName("15677789713");
        user.setUnitId(110);
        user.setHkPersonNo(idGen.nextId());
        user.setUserName("魏熙美测试1");

        // 海康权限信息
        TpUnitHkSetting unitHkSetting = getUnitHkSetting(communityId, user.getUnitId());
        TpEquipmentTree tpEquipmentTree = tpEquipmentTreeMapper.selectByCommunityId(communityId);

        // =================== 添加人员 ===========================
//        Map<String, Object> parUser = Maps.newHashMap();
//        parUser.put("personNo", user.getHkPersonNo());
//        parUser.put("personName", user.getUserName());
//        // parUser.put("phoneNo", user.getLoginName());
//        parUser.put("remark", user.getLoginName());
//        parUser.put("deptUuid", unitHkSetting.getDepartmentId());
//        parUser.put("secret", tpEquipmentTree.getSecret());
//        parUser.put("appkey", tpEquipmentTree.getAppkey());
//        parUser.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
//        parUser.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
//        Map<String, Object> resultMap = JSONObject.parseObject(HKOpenApi.addUser(parUser), HashMap.class);
//        Integer errorCode = (Integer) resultMap.get("errorCode");
//        if (errorCode != 0) {
//            responseBean.addError("人员添加 操作失败");
//            throw new RuntimeException(String.valueOf(resultMap.get("errorMessage")));
//        }
//        Map<String, Object> resultDataMap = (Map<String, Object>) resultMap.get("data");
//        // 海康人员ID
//        Integer personId = (Integer) resultDataMap.get("personId");
//        user.setHkUserId(personId);
//
//        // =================== 添加卡片 =====================
//
//        // 开卡 卡号
//        long cardNo = System.currentTimeMillis();
//        // 添加卡片
//        Map<String,Object> addCardsMap = new HashMap<>();
//        addCardsMap.put("secret", tpEquipmentTree.getSecret());
//        addCardsMap.put("appkey", tpEquipmentTree.getAppkey());
//        addCardsMap.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
//        addCardsMap.put("startCardNo", cardNo);
//        addCardsMap.put("endCardNo", cardNo);
//        addCardsMap.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
//        String addCardsJSON = HKOpenApi.addCards(addCardsMap);
//        JSONObject addCardsJsonObject = JSONObject.parseObject(addCardsJSON);
//        Integer addCardsResultCode = addCardsJsonObject.getInteger("errorCode");
//        if (0 != addCardsResultCode) {
//            responseBean.addError("操作失败");
//            throw new RuntimeException(addCardsJsonObject.getString("errorMessage"));
//        }
//
//        user.setHkCardNo(cardNo + "");
//
//        // ============== 人员绑定卡片 ======================
//
//        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
//        LocalDate today = LocalDate.now();
//        int year = today.getYear();
//        // 加上20, 表示加上20年,默认
//        year += 20;
//
//        int month = today.getMonthValue();
//        int day = today.getDayOfMonth();
//
//        Long time = null;
//        try {
//            time = formatter.parse(year + "-" + month + "-" + day).getTime();
//        } catch (ParseException e) {
//            e.printStackTrace();
//        }
//
//        // 开卡
//        Map<String, Object> parOpenCard = Maps.newHashMap();
//        parOpenCard.put("personId", user.getHkUserId());
//        parOpenCard.put("cardNo", Long.valueOf(cardNo));
//        parOpenCard.put("startTime", System.currentTimeMillis());
//        parOpenCard.put("endTime", time);
//        parOpenCard.put("secret", tpEquipmentTree.getSecret());
//        parOpenCard.put("appkey", tpEquipmentTree.getAppkey());
//        parOpenCard.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
//        parOpenCard.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
//        JSONObject openCardJSON = JSONObject.parseObject(HKOpenApi.openCard(parOpenCard));
//        int code = (int) openCardJSON.get("errorCode");
//        if (code != 0) {
//            String errorMessage = String.valueOf(openCardJSON.get("errorMessage"));
//            responseBean.addError(errorMessage);
//            log.error("人员绑定开卡失败! {}", errorMessage);
//            throw new RuntimeException(errorMessage);
//        }



        // ============= 修改人脸 ==================
        String data = HKOpenApi.HKpersonGroupId(uploadFile, 1236, 1142018896048422912L, user.getUserName(), "1561113497593", tpEquipmentTree.getAppkey(), tpEquipmentTree.getSecret(), tpEquipmentTree.getHttpServer(), tpEquipmentTree.getOpUserUuid());


        //=================== 下发可是对讲卡权限 =====================

        //--------- 可视对讲 ----------

//        Map<String,Object> addOutDoorAuthMap = Maps.newHashMap();
//        // app用户权限勿动 unitHkSetting.getPermissionId()
//        addOutDoorAuthMap.put("authName", "魏熙美测试1");
//        addOutDoorAuthMap.put("personIds", Arrays.asList(user.getHkUserId()));
//        addOutDoorAuthMap.put("secret", tpEquipmentTree.getSecret());
//        addOutDoorAuthMap.put("appkey", tpEquipmentTree.getAppkey());
//        addOutDoorAuthMap.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
//        addOutDoorAuthMap.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
//
//        // 单元门口机
//        List<TpEquipment> visualEquipments = tpEquipmentMapper.selectByCommunityIdAndUnitId(communityId, user.getUnitId(), 3);
//        // 可是对讲设备号
//        List<String> visualList = visualEquipments.stream()
//                .map(e-> tpConfigurationMapper.selectByPrimaryKey(e.getUuidId()).getConfigurationValue())
//                .collect(Collectors.toList());
////        List<String> visualList = new ArrayList<>();
////        visualList.add("10030100001");
//        addOutDoorAuthMap.put("longNums", visualList); // visualList
//        // 可视对讲下发 权限
//         String addOutDoorAuthJSONString = HKOpenApi.addOutDoorAuth(addOutDoorAuthMap);
//        JSONObject addOutDoorAuthJSON = JSONObject.parseObject(addOutDoorAuthJSONString);
//        int addOutDoorAuthJSONCode = (int) addOutDoorAuthJSON.get("errorCode");
//        if (addOutDoorAuthJSONCode != 0 ) {
//            String errorMessage = String.valueOf(addOutDoorAuthJSON.get("errorMessage"));
//            responseBean.addError(errorMessage);
//            log.error("可视对讲下发 权限失败! {}", errorMessage);
//            throw new RuntimeException(errorMessage);
//        }

//        try {
//            Thread.sleep(1000);
//        } catch (InterruptedException e) {
//            e.printStackTrace();
//        }

        // ================ 可是对讲人脸权限下发 ==============

        //可视对讲时 指定人员人脸权限下载
//        List<Integer> userIdList = new ArrayList<>();
//        userIdList.add(user.getHkUserId());
//
//        HKOpenApi.visualIntercom(tpEquipmentTree.getOpUserUuid(), 1, userIdList, visualList, tpEquipmentTree.getAppkey(), tpEquipmentTree.getSecret(), tpEquipmentTree.getHttpServer());
//
//        responseBean.addSuccess("操作成功！");
        return responseBean;
    }
}
